/************************************************************
 *
 * Connection Controller
 *
 ************************************************************/

Ext.define('POSync.controller.Connection', {
    extend: 'Ext.app.Controller',
    id: 'Connection',

    refs: [
        { ref: 'panel', selector: '#panel'},
        { ref: 'projects', selector: '#panel #projects'},
        { ref: 'files', selector: '#panel #files'},
        { ref: 'operations', selector: '#panel #operations'},
        { ref: 'localDir', selector: '#panel #localdir'},
    ],

    /* This is called before the viewport is created */
    init: function() {
        console.log ('Connection.init');
        this.control({
        });
    },

    /* This is called after the viewport is created */
    onLaunch: function() {
        var me = this;
        console.log ('Connection.onLaunch: controller onLaunch');

        this.projectsStore = this.getProjects().getStore();

		this.filesStore = this.getFiles().getStore();
		this.localDirStore = this.getLocalDir().getStore();

        this.operationsStore = Ext.StoreManager.get('Operations');

        var app = this.getApplication();
        this.guiController = app.getController('Gui');

        // receives an even when clicking "sync" on Tray icon
        ipcRenderer.on('sync', (event, arg) => {
                console.log('sync: '+arg);
                me.sync();
        });
    },

    /* FIXME: no easy way to know if login is successful */
    login: function (request_options) {
		var me = this;
        console.log('Connection.login');

        /* Send the login request */
        var configData = this.guiController.configData;
        console.log ('Connection.login: url('+configData.url+') email('+configData.email+') password('+configData.password+')');
        Ext.Ajax.request({
            url: configData.url + '/intranet/auto-login-token',
            params: {
                email: configData.email,
                password: configData.password
            },
            callback: function (options, success, response) {
                if (!success) {
                        console.log('Connection.login: not successfull');
                        // FIXME: Write some message somewhere
                        return;
                }
                var responseJsonString = response.responseText;
                var responseObject = JSON.parse(responseJsonString);
                if (!responseObject) {
                        console.log('Connection.login: error parsing responseText='+responseText);
                        // FIXME: Write some message somewhere
                        return;
                }
                var token = responseObject.token;
                if (!token) {
                        console.log('Connection.login: no token in responseText='+responseText);
                        // FIXME: Write some message somewhere
                        return;
                }
                console.log('Connection.login: token='+token);
                configData.token = token;

                // load the list of projects available to the user
                me.loadProjects();
            }
        });

    },

    /* Send a request to get the list of projects */
    loadProjects: function () {
        var me = this;
        console.log('Connection.loadProjects:');

        // Check if already there
        if (this.projectsStore.isLoaded) return;

        var configData = this.guiController.configData;
        var proxy = this.projectsStore.getProxy();
        var baseUrl = configData.url;

        proxy.url = baseUrl+"/intranet-rest/im_project";
        proxy.extraParams = {
            format:             'json',
            query:              'parent_id is NULL'     // Select only top-level projects
        };

        this.projectsStore.load({
            callback: function (records,operation,success) {
                if (!success) {
                        console.log('Connection.loadProjects: not successfull');
                        // FIXME: Write some message somewhere
                        return;
                }
                me.projectsStore.isLoaded = success;            // mark the store as loaded
            }
        });
    },

    /* Send requests to load the files of all projects */
    loadFiles: function (projectId, successCallback) {
        var me = this;
        console.log('Gui.loadFiles: project_id='+projectId);

        // Skip if already loaded
        //if (''+me.filesStore.isLoaded == ''+projectId) return;

        var configData = this.guiController.configData;
        var proxy = me.filesStore.getProxy();
        var baseUrl = configData.url;
        proxy.url = baseUrl+"/intranet-rest-fs-openacs/file-tree.json";
        proxy.extraParams = {
            format:             'json',
            object_id:          projectId
        };

        me.filesStore.load({
            callback: function (records,operation,success) {
            	if (!success) {
                	console.log('Connection.login: not successfull');
                    // FIXME: Write some message somewhere
                    return;
                }
                me.filesStore.isLoaded = projectId;

				var root = me.filesStore.getRootNode();
			    root.cascadeBy(function(file) {
					file.set('expandable', true);
					//file.set('expanded', true);
				});

                // Execute callback
                successCallback && successCallback.call(me, projectId);
            }
        });

    },

    /*
     * Get the local directory data and load the LocalFiles store.
     * directory-tree provides a tree with the following fields:
     */
    loadLocalDirectory: function (projectId) {
        console.log('Connection.loadLocalDirectory: '+projectId);
        if (!this.projectsStore.isLoaded) { alert('Projects not yet loaded'); return; }

        // Get the project path
        var configData = this.guiController.configData;
        var project = this.projectsStore.getById(''+projectId);
        var path = configData.topdir+'/'+project.get('project_path');

        // make sure the path exists
        var fs = require('fs');
        if (!fs.existsSync(path)) { fs.mkdir(path); }

        // Get the contents
        var dirTree = require ('directory-tree');
        var tree = dirTree(path,{attributes:['mode', 'mtime', 'mtimeMs']});
        this.fixLocalTree(tree);
        this.localDirStore.loadTree (tree);

        this.localDirStore.isLoaded = projectId;
    },

    /* Fix a tree before loading it */
    fixLocalTree: function (tree) {
        if (tree) {
            if (tree.type == 'directory') {
                tree.type = 'folder';
                tree.size = '';
                tree.expandable = true;
                tree.expanded = tree.children && tree.children.length > 0;
                tree.leaf = false;
                tree.mime_type = '';
            } else {
                // const cp = require('child_process');
                // tree.type = cp.execFileSync('/usr/bin/file', ['--brief', '--mime-type', tree.path]).toString();
                // tree.sha1 = cp.execFileSync('/usr/bin/sha1sum', [tree.path]).toString().substr(0,40);
                // tree.date = cp.execFileSync('/usr/bin/stat', ['-c', '%y', tree.path]).toString();

                tree.sha1 = '<sha1>';
                tree.date = '<date>';

                tree.expandable = false;
                tree.expanded = false;
                tree.leaf = true;
                tree.mime_type = tree.type;
            }
            tree.iconCls = 'icon-' + tree.type;
            if (tree.children) {
                var me = this;
                tree.children.forEach (function(child) {
                    me.fixLocalTree (child);
                });
            }
        }
    },

    /**
     * Start the synchronization
     * by loading the files store from the server.
     */
    sync: function (projectId) {
		var me = this;
        console.log('Connection.sync');

		// Load the list of local files needed for sync
		this.loadLocalDirectory(projectId);

	    // Load the list of files and continue with syncCallback
        this.loadFiles(projectId, me.syncFilesLoadedCallback);
	},

    /**
     * The list of files has arrived.
     * Now we can start with the actual sync
     */
	syncFilesLoadedCallback: function(projectId) {
		var me = this;
        console.log('Connection.syncCallback');
        var configData = this.guiController.configData;

		// Clean up the operations store
		me.operationsStore.removeAll();

		// ********************************************************************
	    // Go through the list of remote files and check that a local file exists.
        var root = me.filesStore.getRootNode();
	    root.cascadeBy(function(file) {
                if ("root" == file.get('id')) return;                // Exclude folders and the root

                var pathArray = file.getPathArray();
                var purePathArray = pathArray.slice(0);
                var fileName = file.get('name');
                pathArray.push(fileName);
                console.log('Connection.sync remote: path='+pathArray);

				var type = file.get('type');
				switch (type) {
				case 'file':
                	var localFileFound = me.localDirStore.searchUsingPathArray(pathArray);
                	if (!localFileFound) {
                        var operation = Ext.create('POSync.model.Operation', {
                                id: 'op_'+file.get('id'),
                                file_id: file.get('id'),
                                file_path: purePathArray.join('/'),
                                file_name: file.get('name'),
                                mime_type: file.get('mime_type'),
                                file_sha1: null,
                                op: 'download'
                        });
                        me.operationsStore.add(operation);
					}
                    break;
                case 'folder':
                	var localFileFound = me.localDirStore.searchUsingPathArray(purePathArray);
                	if (!localFileFound) {
                    	// Create a new local folder
        				var fs = require('fs');

        				// Get the project path
        				var project = me.projectsStore.getById(''+projectId);
        				var projectPath = configData.topdir+'/'+project.get('project_path');
						var path = projectPath+'/'+purePathArray.join('/');
        				if (!fs.existsSync(path)) { fs.mkdir(path); }
					}
                    break;
                default: alert('sync: invalid type for remote file: '+type);
                }
        });

		// ********************************************************************
	    // Go through the list of local files and check that a remote file exists
        var root = me.localDirStore.getRootNode();
	    root.cascadeBy(function(file) {
                if ("root" == file.get('id')) return;		// Exclude root

                var pathArray = file.getPathArray();
                var purePathArray = pathArray.slice(0);
                var fileName = file.get('name');
                pathArray.push(fileName);
                console.log('Connection.sync local: path='+pathArray);

				var type = file.get('type');
				switch (type) {
				case 'file':
                	var remoteFileFound = me.filesStore.searchUsingPathArray(pathArray);
                	if (!remoteFileFound) {
                        var operation = Ext.create('POSync.model.Operation', {
                                id: 'op_'+file.get('id'),
                                file_id: file.get('id'),
                                file_path: purePathArray.join('/'),
                                file_name: file.get('name'),
                                mime_type: file.get('mime_type'),
                                file_sha1: null,
                                op: 'upload'
                        });
                        me.operationsStore.add(operation);
					}
                    break;
                case 'folder':
                    var remoteFileFound = me.filesStore.searchUsingPathArray(purePathArray);
                	if (!remoteFileFound) {
                        var operation = Ext.create('POSync.model.Operation', {
                                id: 'op_'+file.get('id'),
                                file_id: file.get('id'),
                                file_path: purePathArray.join('/'),
                                file_name: file.get('name'),
                                op: 'mkdir'
                        });
                        me.operationsStore.add(operation);
					}
                    break;
                default: alert('sync: invalid type for local file: '+type);
                }
        });

        me.syncExecOperations(projectId);
    },


    /**
     * Takes a store with sync "Operations" and executes them.
     * This performs the actual sync.
     */
	syncExecOperations: function(projectId) {
		var me = this;
        console.log('Connection.syncExecOperations');

		me.operationsStore.each(function(op) {
  			console.log('Connection.syncExecOperations: op');
  			console.log(op);

  			var operation = op.get('op');
  			switch (operation) {
			case 'download':
				me.syncDownload(projectId, op);
				break
			case 'upload':
				me.syncUpload(projectId, op);
				break
			case 'mkdir':
				me.syncMkdir(projectId, op);
				break
			default:
				alert('syncExecOperations: Internal error: unknown operation='+operation);
			}
		});

    },

	/**
	 * Download a remote file to the local file-system.
	 *
	 * Called from syncExecOperations, which in turn
	 * loops through the list of operations created during
	 * sync.
	 */
    syncDownload: function(projectId, op) {
		var me = this;
        console.log('Connection.syncDownload');

        var configData = this.guiController.configData;
        var fileId = op.get('file_id');
        var filePath = op.get('file_path');
        var fileName = op.get('file_name');
        var mimeType = op.get('mime_type');

        // Get the project path
        var project = this.projectsStore.getById(''+projectId);
        var projectPath = configData.topdir+'/'+project.get('project_path');
		var path = projectPath+'/'+filePath;

        // make sure the path exists
        var fs = require('fs');
        if (!fs.existsSync(path)) { fs.mkdir(path); }

		var requestUrl = configData.url + '/intranet-rest-fs-openacs/download?file_id='+fileId;
		var xhr = new XMLHttpRequest();
		xhr.open("GET", requestUrl);
		xhr.responseType = "arraybuffer";
		// xhr.responseType = "blob";
		xhr.onload = function () {
			// Asynchronously saving files after they have been downloaded
    		if (this.status === 200) {
            	console.log('Connection.syncDownload: successful download - saving');

				// Decode the response encoded by server in base64
				var responseBuffer = xhr.response;
				var buf64 = new Buffer(responseBuffer, 'base64'); // decode

    		    // make sure the path exists and write the file
				var fs = require('fs');
		        if (!fs.existsSync(path)) { fs.mkdir(path); }
				fs.writeFile(path+"/"+fileName, buf64, function() {});

            	console.log('Connection.syncDownload: written file='+path+'/'+fileName);
    		} else {
            	console.log('Connection.syncDownload: download status='+this.status);
			}
		};
		xhr.send();
	},

	/**
	 * Upload a local file to the remote file-storage
	 */
    syncUpload: function(projectId, op) {
		var me = this;
        console.log('Connection.syncUpload');

        var configData = this.guiController.configData;
        var filePath = op.get('file_path');
        var fileName = op.get('file_name');
        var mimeType = op.get('mime_type');

        // Get the project path
        var project = this.projectsStore.getById(''+projectId);
        var projectPath = configData.topdir+'/'+project.get('project_path');
		var path = projectPath+'/'+filePath;
		var mimeType = 'text/xml';

        // make sure the path exists
        var fs = require('fs');
		var fileContent = fs.readFileSync(path+"/"+fileName);

		// POST the file to the upload page
    	var xhr = new XMLHttpRequest();
    	var url = configData.url + '/intranet-rest-fs-openacs/upload';

		var formData = new FormData();
		formData.append('project_id', projectId);
		formData.append('path', filePath);

		// formData.append('upload_file', fileContent);
		var blob = new Blob([fileContent], { type: mimeType});
		formData.append('upload_file', blob, fileName);


		// xhr.responseType = "arraybuffer";
    	// xhr.submittedData = fileContent;
    	xhr.open("POST", url, true);
		// xhr.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
    	// xhr.setRequestHeader('X-Requested-With','XMLHttpRequest');

    	xhr.onload = function() {
			// Asynchronously saving files after they have been downloaded
 			alert(this.responseText);
    		if (this.status === 200) {
            	console.log('Connection.syncUpload: successful upload');
    		} else {
            	console.log('Connection.syncUpload: upload status='+this.status);
			}
		};
    	xhr.send(formData);
	},

	/**
	 * Create a new folder in the remote file-storage
	 */
    syncMkdir: function(projectId, op) {
		var me = this;
        console.log('Connection.syncMkdir');

		// POST the file to the mkdir page
        var configData = this.guiController.configData;
        var filePath = op.get('file_path');
    	var xhr = new XMLHttpRequest();
    	var url = configData.url + '/intranet-rest-fs-openacs/mkdir';
		var formData = new FormData();
		formData.append('project_id', projectId);
		formData.append('path', filePath);
    	xhr.open("POST", url, true);
    	xhr.onload = function() {
    		if (this.status === 200) {
            	console.log('Connection.syncUpload: successful upload');
    		} else {
 				alert(this.responseText);
            	console.log('Connection.syncUpload: upload status='+this.status+': '+this.responseText);
			}
		};
    	xhr.send(formData);
	}


/* End of POSync.controller.Connection */
});
