/*
 * ]project-open[ filestorage synchronization
 * Copyright (c) 2018 Gonzalo Pérez de Olaguer Córdoba <gonzalo.perez@project-open.com>
 * This code is licensed under the GNU GPL version 2.0 or later
 *
 * This is the electron initialization script.
 * It creates a tray icon to control de application.
 */

const {app, Menu, Tray, BrowserWindow, ipcMain} = require('electron')

let tray = null

// Keep a global reference of the window object, if you don't, the window will
// be closed automatically when the JavaScript object is garbage collected.
let win = null



function createWindow () {
    /* Persistent state */
    const Store = require('electron-store');
    const state_store = new Store({ name: 'state' });
    w = state_store.get('win.width', 1200)
    h = state_store.get('win.height', 600)

    // Create the browser window.
    win = new BrowserWindow({
        title: ']project-open[ filestorage synchronization',
        icon: './images/window.png',
        show: false,
        width: w, height: h
    })

    // and load the index.html of the app.
    win.loadFile('index.html')

    // Open the DevTools.
    win.webContents.openDevTools()

    // Emitted once when the window is ready to be shown.
    // fraber 2018-12-18: Don't show by default
    // win.once('ready-to-show', () => { win.show() })

    // Emitted when the window is closed.
    win.on('closed', () => {
        // Dereference the window object, usually you would store windows
        // in an array if your app supports multi windows, this is the time
        // when you should delete the corresponding element.
        win = null
    })

    // Emitted when the window is resized.
    win.on('resize', () => {
        s = win.getSize()
        state_store.set({win:{width:s[0],height:s[1]}})
    })
}

function showWindow () {
    if (win === null) {
        createWindow()
    } else {
        win.show()
    }
}

function hideWindow () {
    if (win !== null) {
        win.hide()
    }
}

function closeWindow () {
    if (win !== null) {
        win.close()
    }
}

function toggleWindow () {
    if (win === null) {
        createWindow()
    } else if (win.isVisible()) {
        win.hide()
    } else {
        win.show()
    }
}


function sync () {
    if (win === null) {
        createWindow()
    }
    
    // Send an IPC event to the Browser window
    win.webContents.send('sync', 'sync');
    
}


function create_tray_icon () {
    tray = new Tray('./images/tray.png')
    const contextMenu = Menu.buildFromTemplate([
        {label: 'Sync', click: sync},
//        {label: 'Show', click: showWindow},
//        {label: 'Hide', click: hideWindow},
        {label: 'Quit', click: closeWindow}
    ])
    tray.setToolTip(']project-open[ filestorage synchronization')
    tray.setContextMenu(contextMenu)
    tray.on ('click', toggleWindow)
    
    // Create window and start application, but don't show by default
    createWindow();
}

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.on('ready', create_tray_icon)

