
/*
 * DataStore for files.
 *
 * This is a read-only store.
 * Proxy URL depends on application's configuration settings.
 */

Ext.define('POSync.store.Files', {
    storeId:    'Files',
    extend:     'Ext.data.TreeStore',
    requires:   'POSync.model.File',
    model:      'POSync.model.File',
    nodeParam:  'object_id',

    root: {
        expanded: true,
        children: []
    },

    proxy: {
        type:   'ajax',
        url: '/intranet-rest-fs-openacs/file-tree.json',
        reader: {
            type:   'json',
            root:   'children'
        }
    },

    /**
     * Load the store with tree (from local directory)
     */
    loadTree: function (tree) {
		var proxy = this.getProxy();
        proxy.data = tree;
        this.load();
    },

    /**
     * Check if a file with the given path exists
     */
    searchUsingPathArray: function(pathArray) {
        var me = this;
        var root = me.getRootNode();
        // root = root.childNodes[0];		// root is the project's root, so we need to start one below

        // Iterate through the pathArray elements from
        // the top to find the file in the local folder
        var node = root;
        for (var i = 0; i < pathArray.length; i++) {
                var path = pathArray[i];
                // Check all children
                var child = null;
                var children = node.childNodes;
                if (!children) return null;
                for (var j = 0; j < children.length; j++) {
                        var childName = children[j].get('name');
                        if (path === childName) {
                                child = children[j];
                        }
                }

                if (!child) return null;
                node = child;
        }
		return child;
    }


});
