# /packages/intranet-rest-fs-openacs/tcl/intranet-rest-fs-openacs-procs.tcl
#
# Copyright (C) 2018 ]project-open[
#
# All rights reserved. Please check
# http://www.project-open.com/license/ for details.

ad_library {
    Procs used in filestorage API for OpenACS

    @author frank.bergmann@project-open.com
}


# ----------------------------------------------------------------------
# 
# ---------------------------------------------------------------------

ad_proc -public im_rest_fs_folder_for_path {
    -object_id:required
    -path:required
} {
    Returns the ID of the cr_folder related to the given path
    below object
} {
    ns_log Notice "im_rest_fs_folder_for_path: object_id=$object_id, path=$path"

    set folder_id [im_rest_fs_folder_for_object -object_id $object_id]
    ns_log Notice "im_rest_fs_folder_for_path: object_id=$object_id, object_folder_id=$folder_id"

    set path_pieces [split $path "/"]
    foreach piece $path_pieces {
	ns_log Notice "im_rest_fs_folder_for_path: folder_id=$folder_id: searching for '$piece'"
	set folder_id [db_string sub_folder "
		select	min(item_id)
		from	cr_items ci
		where	parent_id = :folder_id and
			lower(name) = lower(:piece)
	"]
    }
    return $folder_id
}


ad_proc -public im_rest_fs_folder_for_object {
    -object_id
} {
    Returns the ID of the cr_folder related to the object
    or creates a new folder.
} {
    set current_user_id [auth::get_user_id]

    set package_id [db_string pid "select package_id from apm_packages where package_key = 'file-storage'" -default ""]
    set package_root_folder_id [fs_get_root_folder -package_id $package_id]

#    ad_return_complaint 1 $root_folder_id

    # Check for first folder related to the object
    set folder_id [db_string fs_folder "
		select	min(ci.item_id)
		from	acs_rels r,
			cr_items ci,
			cr_folders cf
		where	r.object_id_one = :object_id and
			r.object_id_two = ci.item_id and
			r.object_id_two = cf.folder_id	       
    " -default ""]
    if {"" ne $folder_id} { return $folder_id }

    # No folder found:
    # Create a new folder and associate with project

    set object_name [acs_object_name $object_id]
    set object_type [db_string otype "select min(pretty_name) from acs_object_types where object_type in (select object_type from acs_objects where object_id = :object_id)" -default "Object"]
    set folder_name "$object_type: $object_name"
    
    # Check if the folder exists with object type and object name
    # in order to avoid errors with duplicate folder names
    set folder_id [db_string fs_folder "
		select	min(ci.item_id)
		from	cr_items ci,
			cr_folders cf
		where	ci.item_id = cf.folder_id and
			ci.name = :folder_name
    " -default ""]
    
    # Create folder if it doesn't exist
    if {"" eq $folder_id} {
	set folder_id [fs::new_folder \
			       -name $folder_name \
			       -pretty_name $folder_name \
			       -parent_id $package_root_folder_id \
			       -creation_user $current_user_id \
			       -creation_ip [ad_conn peeraddr] \
			       -description $folder_name \
	]

	db_string reg "select content_folder__register_content_type(:folder_id, 'content_folder', true)"
	db_string reg "select content_folder__register_content_type(:folder_id, 'file_storage_object', true)"
	db_string reg "select content_folder__register_content_type(:folder_id, 'content_extlink', true)"
    }

    # Associate the folder with the object
    set rel_id [db_string create_rel "
	    	select acs_rel__new (
			null,
			'im_biz_object_member',
			:object_id,
			:folder_id,
			null,			  
			:current_user_id,
			'[ad_conn peeraddr]'
		)
    "]

    return $folder_id
}


# ----------------------------------------------------------------------
# Components
# ---------------------------------------------------------------------

ad_proc -public im_rest_fs_component {
    -object_id
} {
    Returns a HTML 5 component to show project related files
} {
    # Check if object_id is a sub-project or task
    # and move to top-element. Is is a frequent case...
    set main_project_id [db_string main_p "
	select	main_p.project_id
	from	im_projects main_p,
		im_projects p
	where	p.project_id = :object_id and
		main_p.tree_sortkey = tree_root_key(p.tree_sortkey)
    "]
    if {"" ne $main_project_id} { set object_id $main_project_id }

    # Sencha check and permissions
    if {![im_sencha_extjs_installed_p]} { return "" }
    im_sencha_extjs_load_libraries

    set params [list [list object_id $object_id]]
    # SALO: add file_tree_id
    set file_tree_id "file_tree_[expr {round(rand() * 100000000.0)}]"
    lappend params [list file_tree_id $file_tree_id]
    # SALO: add folder_id
    set folder_id [im_rest_fs_folder_for_object -object_id $object_id]
    lappend params [list folder_id $folder_id]
    set result [ad_parse_template -params $params "/packages/intranet-rest-fs-openacs/lib/file-tree"]
    return [string trim $result]
}

