/*
 * FileTreePanelController.js
 *
 * Copyright (c) 2011 - 2018 ]project-open[ Business Solutions, S.L.
 * This file may be used under the terms of the GNU General Public
 * License version 3.0 or alternatively unter the terms of the ]po[
 * FL or CL license as specified in www.file-open.com/en/license.
 */

/**
 * Deal with collapsible tree nodes, keyboard commands
 * and the interaction with the FileBarPanel.
 */
Ext.define('FileStorage.FileTreePanelController', {
    extend: 'Ext.app.Controller',
    requires: ['Ext.app.Controller'],
    refs: [
        {ref: 'fileBarPanel', selector: '#fileBarPanel'},
        {ref: 'fileTreePanel', selector: '#fileTreePanel'}
    ],

    fileTreePanel: null,

    init: function() {
        var me = this;
        this.control({
            '#fileTreePanel': {
                'itemcollapse': this.onItemCollapse,
                'itemexpand': this.onItemExpand
            },
            '#buttonDelete': { click: this.onButtonDelete}
        });
    },

    /**
     * The user has collapsed a file in the FileTreePanel.
     * We now save the 'c'=closed status using a ]po[ URL.
     * These values will appear in the FileTreeStore.
     */
    onItemCollapse: function(fileModel) {
        var me = this;
        var object_id = fileModel.get('id');
        Ext.Ajax.request({
            url: '/intranet/biz-object-tree-open-close.tcl',
            params: { 'object_id': object_id, 'open_p': 'c' }
        });
    },

    /**
     * The user has expanded a super-file in the FileTreePanel.
     * Please see onItemCollapse for further documentation.
     */
    onItemExpand: function(fileModel) {
        var me = this;
        if (me.debug) console.log('PO.class.FileDrawComponent.onItemExpand: ');

        // Remember the new state
        var object_id = fileModel.get('id');
        Ext.Ajax.request({
            url: '/intranet/biz-object-tree-open-close.tcl',
            params: { 'object_id': object_id, 'open_p': 'o' }
        });
    },

    /**
     * "Delete" (-) button pressed.
     * Delete the currently selected file from the tree.
     */
    onButtonDelete: function() {
        var me = this;
        if (me.debug) console.log('PO.view.file.FileTreePanelController.onButtonDelete: ');

	// ToDo: delete file

    }

});
