# /packages/intranet-rest-fs-openacs/www/file-tree.json.tcl
#
# Copyright (C) 2013-2018 ]project-open[

ad_page_contract {
    Returns a JSON tree structure suitable for batch-loading a project TreeStore
    @param object_id ]po[ Business Object to which the FS belongs. 
           0 identifies the global "Home" FS
    @author frank.bergmann@project-open.com
    
    @param node Passed by ExtJS to load sub-trees of a tree.
                Normally not used, just in case of error.
} {
    {object_id:integer 0}
    {debug_p 0}
    {node ""}
}

set root_file_id $object_id
if {"" ne $node && [string is integer $node]} { set root_file_id $node }

# --------------------------------------------
# Security & Permissions
#
# set current_user_id [auth::require_login]
set current_user_id 624

set ttt {
    im_object_permissions $current_user_id $object_id view read write admin
    if {!$read} {
	im_rest_error -format "json" -http_status 403 -message "You (user #$current_user_id) have no permissions to read object #$object_id"
	ad_script_abort
    }
}


# --------------------------------------------
# Determine folder for object and create if necessary
#
set folder_id [im_rest_fs_folder_for_object -object_id $object_id]

# permission::require_permission -party_id $current_user_id -object_id $folder_id -privilege "read"

# set package_and_root [fs::get_folder_package_and_root $folder_id]
# set package_id [lindex $package_and_root 0]
# set root_folder_id [lindex $package_and_root 1]
set folder_path ""


# --------------------------------------------
# Main hierarchical SQL
#
set top_level [db_string top_level "select tree_level(tree_sortkey) from cr_items where item_id = :folder_id"]

set sql "
	select
		ci.item_id as file_id,
		ci.name as file_name,
		cr.revision_id as version_id,
		tree_level(ci.tree_sortkey) - :top_level as level,
--		encode(digest(cr.content, 'sha1'), 'hex') as sha1,
		(select count(*) from cr_items child where child.parent_id = ci.item_id) as num_children,
		ci.*,
		cr.*,
		CASE WHEN bts.open_p = 'o' THEN 'true' ELSE 'false' END as expanded,
		CASE
			WHEN ci.content_type::text = 'content_folder' THEN 'folder'
			WHEN ci.content_type::text = 'content_extlink' THEN 'url'
			WHEN ci.content_type::text = 'content_symlink' THEN 'symlink'
			WHEN ci.content_type::text = 'file_storage_object' THEN 'file'
			ELSE ci.content_type
		END AS type
	from	cr_items parent_ci,
		cr_items ci
		LEFT JOIN cr_extlinks ce ON ci.item_id = ce.extlink_id
		LEFT JOIN cr_folders cf ON ci.item_id = cf.folder_id
		LEFT JOIN cr_revisions cr ON ci.live_revision = cr.revision_id
		LEFT JOIN cr_mime_types cm ON cr.mime_type = cm.mime_type
		JOIN acs_objects o ON ci.item_id = o.object_id
		LEFT OUTER JOIN im_biz_object_tree_status bts ON (
			ci.item_id = bts.object_id and 
			bts.page_url = 'default' and
			bts.user_id = :current_user_id
		)
	where
		parent_ci.item_id = :folder_id and
		ci.tree_sortkey between parent_ci.tree_sortkey and tree_right(parent_ci.tree_sortkey)
		and tree_level(ci.tree_sortkey) > :top_level		
--		and tree_level(ci.tree_sortkey) <= :top_level+1
	order by
		ci.tree_sortkey
"
# ad_return_complaint 1 [im_ad_hoc_query -subtotals_p 0 -format html $sql]


# Read the query into a Multirow, so that we can order
# it according to sort_order within the individual sub-levels.
db_multirow file_multirow file_list $sql {
	# By default open the top-level file-storage
	if {0 eq $level} { set expanded "true" }
}


# --------------------------------------------
# Get all the extra variables for a file ci_item:
# fraber 2018-08-22: No meta-variables yet...
set valid_vars [list level version_id type publish_date mime_type content_length sha1 title description]
set valid_vars [lsort -unique $valid_vars]

# needed to calculate sha1
set storage_area_key "CR_FILES"
set storage_area_path [cr_fs_path $storage_area_key]
ns_log Debug "file-tree.json.tcl: storage_area_key($storage_area_key) storage_area_path($storage_area_path)"

set file_json ""
set ctr 0
set old_level 0
set indent ""
template::multirow foreach file_multirow {

	ns_log Notice "file-tree.json.tcl: file_id=$file_id, file_id=$file_id, version_id=$version_id, name($file_name)"
	if {$debug_p} { append file_json "\n// finish: ctr=$ctr, level=$level, old_level=$old_level\n" }

	# -----------------------------------------
	# Close off the previous entry
	# -----------------------------------------
	
	# This is the first child of the previous item
	# Increasing the level always happens in steps of 1
	if {$level > $old_level} {
	append file_json ",\n${indent}\t\"children\":\[\n"
	}

	# A group of children needs to be closed.
	# Please note that this can cascade down to several levels.
	while {$level < $old_level} {
	append file_json "\n${indent}\}\]\n"
	incr old_level -1
	set indent ""
	for {set i 0} {$i < $old_level} {incr i} { append indent "\t" }
	}

	# The current file is on the same level as the previous.
	# This is also executed after reducing the old_level in the previous while loop
	if {$level == $old_level} {
	if {0 != $ctr} { 
		append file_json "${indent}\n${indent}\},\n"
	}
	}

	if {$debug_p} { append file_json "\n// $file_name: ctr=$ctr, level=$level, old_level=$old_level\n" }

	set indent ""
	for {set i 0} {$i < $level} {incr i} { append indent "\t" }
	
	if {0 == $num_children} { set leaf_json "true" } else { set leaf_json "false" }
	if {"folder" == $type} { set leaf_json "false" }

	set quoted_char_map {"\n" "\\n" "\r" "\\r" "\"" "\\\"" "\\" "\\\\"}
	set quoted_file_name [string map $quoted_char_map $file_name]
	
	append file_json "${indent}\{
${indent}\t\"id\":$file_id,
${indent}\t\"name\":\"$quoted_file_name\",
${indent}\t\"iconCls\":\"icon-$type\",
${indent}\t\"expanded\":$expanded,
"

	if {"folder" == $type && 0 == $num_children} {
	    append file_json "${indent}\t\"children\":\[\],\n"
	}
	
	# version_id is empty for directories
	if {"" eq $version_id} {
	    set sha1 ""
	} else {
	    set filename [db_string filename "select :storage_area_path || content from cr_revisions where revision_id = :version_id"]
	    set sha1 [string range [exec /usr/bin/sha1sum "$filename"] 0 39]
	    ns_log Debug "file-tree.json.tcl: version_id($version_id) filename($filename) sha1($sha1)"
	}

	foreach var $valid_vars {
	    # Skip xml_* variables (only used by MS-File)
	    if {[regexp {^xml_} $var match]} { continue }
	    
	    # Append the value to the JSON output
	    set value [set $var]
	    set quoted_value [string map $quoted_char_map $value]
	    append file_json "${indent}\t\"$var\":\"$quoted_value\",\n"
	}
	append file_json "${indent}\t\"leaf\":$leaf_json"

	incr ctr
	set old_level $level
}

set level 0
while {$level < $old_level} {
	# A group of children needs to be closed.
	# Please note that this can cascade down to several levels.
	append file_json "\n${indent}\}\]\n"
	incr old_level -1
	set indent ""
	for {set i 0} {$i < $old_level} {incr i} { append indent "\t" }
}
