ad_page_contract {
    page to add a new file to the system

    @author Frank Bergmann (frank.bergmann@project-open.com)
    @creation-date 6 Nov 2018
    @cvs-id $Id$
} {
    project_id:integer
    { path ""}
    upload_file:trim,optional
    upload_file.tmpfile:tmpfile,optional
    { title "" }
    { description "description" }
    { creation_date "" }
    { modification_date "" }
    { sha1 "" }
}

set user_id [ad_conn user_id]
set n_bytes [file size ${upload_file.tmpfile}]
set package_id [ad_conn package_id]
set package_id [db_string package "select min(package_id) from apm_packages where package_key = 'file-storage'" -default $package_id]

ns_log Notice "upload: project_id=$project_id, path=$path, upload_file=$upload_file, tmp_file=${upload_file.tmpfile}"

set max_bytes [parameter::get_from_package_key -package_key "file-storage" -parameter "MaximumFileSize"]
if {$n_bytes > $max_bytes } {
    doc_return 507 "application/json" "{success: false, message: \"File is larger than the maximum size ($max_bytes bytes) allowed on this server\"}"
    ad_script_abort
}

set folder_id [im_rest_fs_folder_for_path -object_id $project_id -path $path]
if {"" eq $folder_id} {
    # Folder path not found
    doc_return 400 "application/json" "{'success': false, 'message': 'Did not find folder ´[ns_quotehtml $path]´', 'data': {}}"
    ad_script_abort
}

set mime_type [cr_filename_to_mime_type -create -- $upload_file]
set file_id [db_nextval "acs_object_id_seq"]
ns_log Notice "upload: folder_id=$folder_id, file_id=$file_id, mime_type=$mime_type"


set file_id [fs::add_file \
        -name $upload_file \
        -item_id $file_id \
        -parent_id $folder_id \
        -tmp_filename ${upload_file.tmpfile} \
        -creation_user $user_id \
        -creation_ip [ad_conn peeraddr] \
        -title $upload_file \
        -description $description \
        -package_id $package_id \
        -mime_type $mime_type \
]


file delete ${upload_file.tmpfile}

doc_return 200 "application/json" "{success: true, message: \"Upload successful.\", data: {id: $file_id}}"
ad_script_abort

